@@ -5,6 +5,8 @@ it may become separate library in the future. Things like imports and logging sh
 import base64
 import json
 import logging
+import math
+import os
 import time
 import traceback
 from collections import OrderedDict
@@ -14,8 +16,10 @@ from urllib.parse import urlencode
 import requests
 
 from bzt import ManualShutdown, TaurusException, TaurusNetworkError
+from bzt.engine import ScenarioExecutor
+from bzt.jmx import try_convert
 from bzt.resources.version import VERSION
-from bzt.utils import to_json, MultiPartForm, NETWORK_PROBLEMS
+from bzt.utils import to_json, MultiPartForm, NETWORK_PROBLEMS, dehumanize_time
 
 BZA_TEST_DATA_RECEIVED = 100
 ENDED = 140
@@ -79,6 +83,7 @@ class BZAObject(dict):
         self.http_session = requests.Session()
         self.http_request = self.http_session.request
         self._retry_limit = 5
+        self.anonymous_access_blocked = False
 
         # copy infrastructure from prototype
         if isinstance(proto, BZAObject):
@@ -175,6 +180,32 @@ class BZAObject(dict):
 
         return result
 
+    def _get_load_from_config(self, config):
+        def eval_int(value):
+            try:
+                return int(value)
+            except (ValueError, TypeError, KeyError):
+                return value
+
+        concurrency = 1
+        ramp_up = 0
+        hold = 0
+        duration = 1
+        if 'execution' in config and config['execution'] and config['execution'][0]:
+            if 'concurrency' in config['execution'][0] \
+                    and config['execution'][0]['concurrency'] and 'provisioning' in config:
+                concurrency = eval_int(
+                    config['execution'][0]['concurrency'][config['provisioning']] or 0)
+            if ScenarioExecutor.RAMP_UP in config['execution'][0]:
+                ramp_up = try_convert(eval_int(config['execution'][0][ScenarioExecutor.RAMP_UP] or "0m"),
+                                      dehumanize_time, 0)
+            if ScenarioExecutor.HOLD_FOR in config['execution'][0]:
+                hold = try_convert(eval_int(config['execution'][0][ScenarioExecutor.HOLD_FOR] or "0m"),
+                                   dehumanize_time, 0)
+        if (ramp_up + hold) > 0:
+            duration = math.ceil((ramp_up + hold) / 60)
+        return concurrency, duration
+
 
 class BZAObjectsList(list):
     def first(self):
@@ -266,54 +297,91 @@ class User(BZAObject):
 class Account(BZAObject):
     def workspaces(self, ident=None, name=None):
         """
-        :rtype: BZAObjectsList[Workspace]
-        """
-        params = {"accountId": self['id'], 'enabled': 'true', 'limit': 100}
-        params = OrderedDict(sorted(params.items(), key=lambda t: t[0]))
-        res = self._request(self.address + '/api/v4/workspaces?' + urlencode(params))
+                :rtype: BZAObjectsList[Workspace]
+                """
+        # Initialize parameters
+        if ident:
+            return self.workspace(name=name, ident=ident)
+        params = {"accountId": self['id'], 'enabled': 'true'}
+        # Pagination settings
+        limit = 100
+        skip = 0
+        total = None
+        # List to store the results
         workspaces = []
-        for wksp in res['result']:
-            if not wksp['enabled']:
-                continue
+        # Pagination loop
+        while total is None or skip < total:
+            params.update({"limit": limit, "skip": skip})
+            response = self._request(f"{self.address}/api/v4/workspaces?{urlencode(params)}")
+            # Extract results and total count
+            total = response.get("total", 0)
+            results = response.get('result', [])
 
-            if name is not None and wksp['name'] != name:
-                continue
+            # Filter and add workspaces
+            for workspace in results:
+                if workspace['enabled'] and (name is None or workspace['name'] == name):
+                    workspaces.append(Workspace(self, workspace))
 
-            if ident is not None and wksp['id'] != ident:
-                continue
-
-            workspaces.append(Workspace(self, wksp))
+            skip += limit
 
         return BZAObjectsList(workspaces)
 
+    def workspace(self, name=None, ident=None):
+        workspaces = BZAObjectsList()
+        response = self._request(f"{self.address}/api/v4/workspaces/{ident}")
+        result = response.get('result', None)
+        if result and (name is None or result['name'] == name):
+            workspaces.append(Workspace(self, result))
+        return workspaces
 
 class Workspace(BZAObject):
     def projects(self, name=None, ident=None):
         """
-        :rtype: BZAObjectsList[Project]
-        """
-        params = OrderedDict()
-        params.update({"workspaceId": self['id']})
+                :rtype: BZAObjectsList[Project]
+                """
+        if ident:
+            return self.project(name=name, ident=ident)
+        # Initialize parameters
+        params = {"workspaceId": self['id']}
 
         if name:
-            params.update({"name": name})
+            params["name"] = name
 
-        if ident:
-            params.update({"limit": 1000})
-
-        res = self._request(self.address + '/api/v4/projects?' + urlencode(params))
+        # Pagination settings
+        limit = 100
+        skip = 0
+        total = None
 
+        # List to store projects
         projects = BZAObjectsList()
-        for item in res['result']:
-            if name is not None and item['name'] != name:
-                continue
 
-            if ident is not None and item['id'] != ident:
-                continue
+        # Pagination loop
+        while total is None or skip < total:
+            params.update({"limit": limit, "skip": skip})
+
+            response = self._request(f"{self.address}/api/v4/projects?{urlencode(params)}")
+            # Extract results and total count
+            total = response.get("total", 0)
+            results = response.get('result', [])
+
+            # Filter and add projects
+            for item in results:
+                if name is None or item['name'] == name:
+                    projects.append(Project(self, item))
+
+            skip += limit
 
-            projects.append(Project(self, item))
         return BZAObjectsList(projects)
 
+    def project(self, name=None, ident=None):
+        projects = BZAObjectsList()
+        response = self._request(f"{self.address}/api/v4/projects/{ident}")
+        result = response.get('result', None)
+        if result and (name is None or result['name'] == name) \
+                and result['workspaceId'] == self['id']:
+            projects.append(Project(self, result))
+        return projects
+
     def locations(self, include_private=False):
         if 'locations' not in self:
             self.fetch()
@@ -336,25 +404,46 @@ class Workspace(BZAObject):
         """
         :rtype: BZAObjectsList[Test]
         """
-        params = OrderedDict({"workspaceId": self['id']})
-        if name is not None:
+        if ident:
+            return self.test(name=name, ident=ident, test_type=test_type)
+        params = {"workspaceId": self['id']}
+        if name:
             params["name"] = name
-        if ident is not None:
-            params["id"] = ident
 
-        res = self._request(self.address + '/api/v4/tests?' + urlencode(params))
+        # Pagination settings
+        limit = 100
+        skip = 0
+        total = None
+
+        # List to store tests
         tests = BZAObjectsList()
-        for item in res['result']:
-            if ident is not None and item['id'] != ident:
-                continue
 
-            if name and item['name'] != name:
-                continue
+        # Pagination loop
+        while total is None or skip < total:
+            params.update({"limit": limit, "skip": skip})
+            response = self._request(f"{self.address}/api/v4/tests?{urlencode(params)}")
+            # Extract results and total count
+            total = response.get("total", 0)
+            results = response.get('result', [])
 
-            if test_type and item['configuration']['type'] != test_type:
-                continue
+            # Filter and add tests
+            for item in results:
+                if (name is None or item['name'] == name) and (
+                        test_type is None or item['configuration']['type'] == test_type):
+                    tests.append(Test(self, item))
 
-            tests.append(Test(self, item))
+            skip += limit
+
+        return tests
+
+    def test(self, name=None, ident=None, test_type=None):
+        response = self._request(f"{self.address}/api/v4/tests/{ident}")
+        result = response.get('result', None)
+
+        tests = BZAObjectsList()
+        if result and (name is None or result['name'] == name) \
+                and (test_type is None or result['configuration']['type'] == test_type):
+            tests.append(Test(self, result))
         return tests
 
     def create_project(self, proj_name):
@@ -377,27 +466,49 @@ class Project(BZAObject):
         """
         :rtype: BZAObjectsList[Test]
         """
-        params = OrderedDict({"projectId": self['id']})
-        if name is not None:
+        # Initialize parameters
+        if ident:
+            return self.test(name=name, ident=ident, test_type=test_type)
+        params = {"projectId": self['id']}
+        if name:
             params["name"] = name
-        if ident is not None:
-            params["id"] = ident
 
-        res = self._request(self.address + '/api/v4/tests?' + urlencode(params))
+        # Pagination settings
+        limit = 100
+        skip = 0
+        total = None
+
         tests = BZAObjectsList()
-        for item in res['result']:
-            if ident is not None and item['id'] != ident:
-                continue
+        # Pagination loop
+        while total is None or skip < total:
+            params.update({"limit": limit, "skip": skip})
+            response = self._request(f"{self.address}/api/v4/tests?{urlencode(params)}")
+            # Extract results and total count
+            total = response.get("total", 0)
+            results = response.get('result', [])
 
-            if name is not None and item['name'] != name:
-                continue
+            # Filter and add tests
+            for item in results:
+                if (name is None or item['name'] == name) and (
+                        test_type is None or item['configuration']['type'] == test_type):
+                    tests.append(Test(self, item))
 
-            if test_type is not None and item['configuration']['type'] != test_type:
-                continue
+            skip += limit
 
-            tests.append(Test(self, item))
         return tests
 
+    def test(self, name=None, ident=None, test_type=None):
+        response = self._request(f"{self.address}/api/v4/tests/{ident}")
+        result = response.get('result', None)
+
+        tests = BZAObjectsList()
+        if result and (name is None or result['name'] == name) \
+                and (test_type is None or result['configuration']['type'] == test_type)\
+                and (result.get('projectId') == self['id']):
+            tests.append(Test(self, result))
+        return tests
+
+
     def create_test(self, name, configuration):
         """
         :param name:
@@ -412,21 +523,39 @@ class Project(BZAObject):
 
 
 class Test(BZAObject):
-    def start_external(self):
+    def start_external(self, config):
+        concurrency, duration = self._get_load_from_config(config)
+        data = {
+            "concurrency": concurrency,
+            "duration": duration
+        }
+        data_str = json.dumps(data)
         url = self.address + "/api/v4/tests/%s/start-external" % self['id']
-        res = self._request(url, method='POST')
+        res = self._request(url, data_str, method='POST')
         result = res['result']
+        if 'warning' in result and result['warning']:
+            self.anonymous_access_blocked = True
+            self.log.warning(result['warning'])
         session = Session(self, result['session'])
         session.data_signature = result['signature']
         return session, Master(self, result['master'])
 
-    def start_anonymous_external_test(self):
+    def start_anonymous_external_test(self, config):
         """
         :rtype: (Session,Master,str)
         """
+        concurrency, duration = self._get_load_from_config(config)
+        data = {
+            "concurrency": concurrency,
+            "duration": duration
+        }
+        data_str = json.dumps(data)
         url = self.address + "/api/v4/sessions"
-        res = self._request(url, method='POST')
+        res = self._request(url, data_str, method='POST')
         result = res['result']
+        if 'warning' in result and result['warning']:
+            self.anonymous_access_blocked = True
+            self.log.warning(result['warning'])
         session = Session(self, result['session'])
         session.data_signature = result['signature']
         return session, Master(self, result['master']), result['publicTokenUrl']
@@ -478,7 +607,7 @@ class Test(BZAObject):
         res = self._request(url, data=coll, method="PATCH")
         return res['result']
 
-    def started_passfail_validation(self):
+    def started_taurus_yml_validation(self):
         # validate passfail configuration, get success if started
         url = f"{self.address}/api/v4/tests/{self['id']}/validate"
         resp = self._request(url, data={"files": [{"fileName": "taurus.yml"}], "performDataMerge": False})
@@ -487,18 +616,18 @@ class Test(BZAObject):
             return result['success']
         return False
 
-    def get_passfail_validation(self):
-        # get passfail validation status and results, log warnings if present
+    def get_taurus_yml_validation(self):
+        # get validation status and results, log warnings if present
         url = f"{self.address}/api/v4/tests/{self['id']}/validations"
         resp = self._request(url, method='GET')
         validated = False
         for result in resp.get('result'):
             if result['fileName'] == 'taurus.yml' and result['status'] == 100:
                 for warning_msg in result['warnings'] + result['fileWarnings']:
-                    self.log.warning(f"Passfail Warning: {warning_msg}")
+                    self.log.warning(f"Validation warning: {warning_msg}")
                 validated = True
         if not validated:
-            self.log.debug(f"Passfail error: Unable to validate by {url}.")
+            self.log.debug(f"Validation error: Unable to validate by {url}.")
         return validated
 
 
@@ -603,6 +732,12 @@ class Master(BZAObject):
         url = self.address + "/api/v4/masters/%s/reports/functional/groups/%s" % (self['id'], group_id)
         return self._request(url)['result']
 
+    def get_csv_report(self, file_name):
+        file_name_without_extension = os.path.splitext(file_name)[0]
+        url = self.address + "/api/v4/data/dump?masterId=%s&fileName=%s" % (self['id'], file_name_without_extension)
+        res = self._request(url)
+        return res['result']
+
 
 class Session(BZAObject):
     def __init__(self, proto=None, data=None):
@@ -644,6 +779,9 @@ class Session(BZAObject):
         :param is_check_response:
         :type data: str
         """
+        if self.anonymous_access_blocked:
+            logging.debug("Data submit is blocked, ignoring sending KPI")
+            return
         submit_target = self.kpi_target if submit_target is None else submit_target
         url = self.data_address + "/submit.php?session_id=%s&signature=%s&test_id=%s&user_id=%s"
         url %= self['id'], self.data_signature, self['testId'], self['userId']
@@ -679,8 +817,11 @@ class Session(BZAObject):
         :type contents: str
         :raise TaurusNetworkError:
         """
-        body = MultiPartForm()
+        if self.anonymous_access_blocked:
+            logging.debug("Data submit is blocked, ignoring uploading files")
+            return
 
+        body = MultiPartForm()
         if contents is None:
             body.add_file('file', filename)
         else:
