@@ -18,21 +18,6 @@ limitations under the License.
 from abc import abstractmethod
 
 
-class FileLister(object):
-    """
-    A mixin to get required files info from executor
-    """
-
-    @abstractmethod
-    def resource_files(self):
-        """
-        Get list of resource files
-
-        :rtype: list
-        """
-        pass
-
-
 class HavingInstallableTools(object):
     @abstractmethod
     def install_required_tools(self):
@@ -41,13 +26,3 @@ class HavingInstallableTools(object):
 
 class Singletone(object):
     pass
-
-
-class SelfDiagnosable(object):
-    @abstractmethod
-    def get_error_diagnostics(self):
-        """
-
-        :rtype: list[str]
-        """
-        pass
