@@ -113,6 +113,15 @@ class JMX(object):
             self.tree = etree.ElementTree()
             self.tree.parse(original)
         except BaseException as exc:
+            exc_str = str(exc)
+            if isinstance(exc, etree.XMLSyntaxError) and ("huge text node" in exc_str or "XML_PARSE_HUGE" in exc_str):
+                self.log.warning("XML parsing failed due to huge text node, retrying with allowed huge text nodes")
+                try:
+                    # Retry with allowed huge text nodes
+                    return self.tree.parse(original, parser=etree.XMLParser(huge_tree=True))
+                except BaseException as exc2:
+                    msg = "XML parsing (with huge_tree) failed for file %s: %s"
+                    raise TaurusInternalException(msg % (original, exc))
             msg = "XML parsing failed for file %s: %s"
             raise TaurusInternalException(msg % (original, exc))
 
@@ -562,7 +571,8 @@ class JMX(object):
 
     @staticmethod
     def get_thread_group(concurrency=None, rampup=0, hold=0, iterations=None,
-                         testname="ThreadGroup", on_error="continue", thread_delay=False, scheduler_delay=None):
+                         testname="ThreadGroup", on_error="continue", thread_delay=False, scheduler_delay=None,
+                         add_ramp_to_duration=True, same_user=True):
         """
         Generates ThreadGroup
 
@@ -573,6 +583,7 @@ class JMX(object):
             ThreadGroup.duration (rampup + hold): int
             LoopController.loops (iterations): int
             ThreadGroup.delayedStart: boolean
+            ThreadGroup.same_user_on_next_iteration: boolean
         :return: etree element, ThreadGroup
         """
         rampup = cond_int(rampup or 0)
@@ -592,6 +603,8 @@ class JMX(object):
             duration = hold
         elif isinstance(rampup, numeric_types) and isinstance(hold, numeric_types):
             duration = hold + rampup
+        elif not add_ramp_to_duration:
+            duration = hold
         else:
             duration = "${__intSum(%s,%s)}" % (rampup, hold)
 
@@ -614,7 +627,8 @@ class JMX(object):
                              name="ThreadGroup.main_controller",
                              elementType="LoopController",
                              guiclass="LoopControlPanel",
-                             testclass="LoopController")
+                             testclass="LoopController",
+                             testname="Loop Controller")
 
         # 'true' causes endless execution of TG in non-gui mode
         loop.append(JMX._bool_prop("LoopController.continue_forever", False))
@@ -627,6 +641,7 @@ class JMX(object):
         trg.append(JMX._string_prop("ThreadGroup.end_time", ""))
         trg.append(JMX._bool_prop("ThreadGroup.scheduler", scheduler))
         trg.append(JMX._string_prop("ThreadGroup.duration", duration))
+        trg.append(JMX._bool_prop("ThreadGroup.same_user_on_next_iteration", same_user))
 
         if scheduler_delay:
             trg.append(JMX._string_prop("ThreadGroup.delay", scheduler_delay))
@@ -729,7 +744,7 @@ class JMX(object):
 
     @staticmethod
     def get_concurrency_thread_group(concurrency=None, rampup=0, hold=0, steps=None, on_error="continue",
-                                     testname="ConcurrencyThreadGroup", iterations=""):
+                                     testname="ConcurrencyThreadGroup", iterations="", same_user=True):
         """
         Generates ConcurrencyThreadGroup
 
@@ -771,6 +786,7 @@ class JMX(object):
         concurrency_thread_group.append(JMX._string_prop("LogFilename", ""))
         concurrency_thread_group.append(JMX._string_prop("Iterations", iterations or ""))
         concurrency_thread_group.append(JMX._string_prop("Unit", "S"))
+        concurrency_thread_group.append(JMX._bool_prop("same_user_on_next_iteration", same_user))
 
         return concurrency_thread_group
 
