@@ -146,8 +146,17 @@ class LoadSettingsProcessor(object):
             existed_tg = (not is_jmx_generated) and (group.gtype == self.TG)
             if not self.force_ctg and existed_tg:
                 iterations = group.get_iterations()
+
+            if self.raw_load.ramp_up:
+                ramp_up = self.load.ramp_up
+                add_ramp_to_duration = True
+            else:
+                ramp_up = group.get_ramp_up()
+                add_ramp_to_duration = False
+
             self.tg_handler.convert(source=group, target_gtype=self.tg, load=self.load,
-                                    concurrency=concurrency, iterations=iterations)
+                                    concurrency=concurrency, ramp_up=ramp_up, iterations=iterations,
+                                    add_ramp_to_duration=add_ramp_to_duration)
         if self.load.throughput:
             self._add_shaper(jmx)
 
@@ -192,8 +201,8 @@ class LoadSettingsProcessor(object):
         if self.load.ramp_up:
             if isinstance(self.load.throughput, numeric_types) and self.load.duration:
                 start_rps = self.load.throughput / float(self.load.duration)
-                start_rps = max(start_rps, 0.001)  # avoid zeroing
-                start_rps = min(start_rps, 1.0)  # avoid starting too fast
+                if start_rps < 1.0:
+                    start_rps = 1.0  # avoid zeroing
             else:
                 start_rps = 1
 
