@@ -22,14 +22,14 @@ from math import ceil
 from distutils.version import LooseVersion
 
 from bzt import TaurusConfigError
-from bzt.engine import ScenarioExecutor, HavingInstallableTools, SelfDiagnosable
+from bzt.engine import ScenarioExecutor
 from bzt.modules.aggregator import ConsolidatingAggregator, ResultsReader
-from bzt.modules.console import WidgetProvider, ExecutorWidget
+from bzt.modules.console import ExecutorWidget
 from bzt.requests_model import HTTPRequest
 from bzt.utils import iteritems, CALL_PROBLEMS, shutdown_process, RequiredTool, dehumanize_time, FileReader
 
 
-class ApacheBenchmarkExecutor(ScenarioExecutor, WidgetProvider, HavingInstallableTools, SelfDiagnosable):
+class ApacheBenchmarkExecutor(ScenarioExecutor):
     """
     Apache Benchmark executor module
     """
@@ -64,7 +64,7 @@ class ApacheBenchmarkExecutor(ScenarioExecutor, WidgetProvider, HavingInstallabl
         """
         if not self.widget:
             label = "%s" % self
-            self.widget = ExecutorWidget(self, "ab: " + label.split('/')[1])
+            self.widget = ExecutorWidget(self, "AB: " + label.split('/')[1])
         return self.widget
 
     def __first_http_request(self):
@@ -121,6 +121,10 @@ class ApacheBenchmarkExecutor(ScenarioExecutor, WidgetProvider, HavingInstallabl
         if request.priority_option('keepalive', default=True):
             args += ['-k']
 
+        user_cmd = self.settings.get("cmdline")
+        if user_cmd:
+            args += user_cmd.split(" ")
+
         args += [request.url]
 
         self.reader.setup(load_concurrency, request.label)
