@@ -25,6 +25,7 @@ import traceback
 import zipfile
 from collections import defaultdict, OrderedDict
 from io import BytesIO
+from typing import List
 from urllib.error import HTTPError
 
 import requests
@@ -32,11 +33,14 @@ import requests
 from bzt import TaurusInternalException, TaurusConfigError, TaurusNetworkError
 from bzt.bza import User, Session, Test
 from bzt.engine import Reporter, Singletone
-from bzt.utils import b, humanize_bytes, iteritems, open_browser, BetterDict, to_json, dehumanize_time
-from bzt.modules.aggregator import AggregatorListener, DataPoint, KPISet, ResultsProvider, ConsolidatingAggregator
-from bzt.modules.monitoring import Monitoring, MonitoringListener
-from bzt.modules.blazemeter.project_finder import ProjectFinder
+from bzt.modules.aggregator import AggregatorListener, DataPoint, KPISet, ResultsProvider
 from bzt.modules.blazemeter.const import NOTE_SIZE_LIMIT
+from bzt.modules.blazemeter.engine_metrics import HappysocksMetricsConverter, MetricsReportingBuffer, \
+    HappySocksConcurrencyConverter
+from bzt.modules.blazemeter.project_finder import ProjectFinder
+from bzt.modules.monitoring import Monitoring, MonitoringListener
+from bzt.utils import b, humanize_bytes, iteritems, open_browser, BetterDict, to_json, dehumanize_time, \
+    HappysocksClient, HappysocksEngineNamespace
 
 
 class BlazeMeterUploader(Reporter, AggregatorListener, MonitoringListener, Singletone):
@@ -50,21 +54,34 @@ class BlazeMeterUploader(Reporter, AggregatorListener, MonitoringListener, Singl
 
     def __init__(self):
         super(BlazeMeterUploader, self).__init__()
+        self.send_concurrency_metric = False
         self.browser_open = 'start'
         self.kpi_buffer = []
+        self._engine_metrics_send_interval = 5
         self.send_interval = 30
         self._last_status_check = time.time()
         self.send_data = True
         self.upload_artifacts = True
         self.send_monitoring = True
         self.monitoring_buffer = None
+        self._engine_metrics_buffer = None
+        self._concurrency_buffer = None
+        self._sess_id = None
+        self._master_id = None
+        self._calibration_id = None
+        self._calibration_step_id = None
         self.public_report = False
         self.last_dispatch = 0
+        self._last_metrics_dispatch = 0
+        self._happysocks_address = None
         self.results_url = None
         self._user = User()
         self._test = None
         self._master = None
         self._session = None
+        self.happysocks_client = None
+        # when true, happysocks client handles reconnect by itself
+        self._happysocks_auto_reconnect = False
         self.first_ts = sys.maxsize
         self.last_ts = 0
         self.report_name = None
@@ -80,6 +97,13 @@ class BlazeMeterUploader(Reporter, AggregatorListener, MonitoringListener, Singl
         self.send_monitoring = self.settings.get("send-monitoring", self.send_monitoring)
         monitoring_buffer_limit = self.settings.get("monitoring-buffer-limit", 500)
         self.monitoring_buffer = MonitoringBuffer(monitoring_buffer_limit, self.log)
+        self._engine_metrics_buffer = MetricsReportingBuffer(monitoring_buffer_limit, True)
+        self._concurrency_buffer = MetricsReportingBuffer(monitoring_buffer_limit, False)
+        self._engine_metrics_send_interval = dehumanize_time(
+            self.settings.get("engine-metrics-send-interval", self._engine_metrics_send_interval))
+        happysocks_verbose_logging = self.settings.get("happysocks-verbose-logging", False)
+        happysocks_verify_ssl = self.settings.get("happysocks-verify-ssl", True)
+        self.send_concurrency_metric = self.settings.get("send-concurrency-metric", False)
         self.browser_open = self.settings.get("browser-open", self.browser_open)
         self.public_report = self.settings.get("public-report", self.public_report)
         self.upload_artifacts = self.parameters.get("upload-artifacts", self.upload_artifacts)
@@ -99,15 +123,20 @@ class BlazeMeterUploader(Reporter, AggregatorListener, MonitoringListener, Singl
             self._user.http_session = self.engine.get_http_client()
             self._user.http_request = self._user.http_session.request
 
+        self._calibration_id = self.parameters.get("calibration-id")
+        self._calibration_step_id = self.parameters.get("calibration-step-id")
         # direct data feeding case
-        sess_id = self.parameters.get("session-id")
-        if sess_id:
-            self._session = Session(self._user, {'id': sess_id})
+        self._master_id = self.parameters.get("master-id")
+        self._sess_id = self.parameters.get("session-id")
+        signature = None
+        if self._sess_id:
+            self._session = Session(self._user, {'id': self._sess_id})
             self._session['userId'] = self.parameters.get("user-id", None)
             self._session['testId'] = self.parameters.get("test-id", None)
             self._test = Test(self._user, {'id': self._session['testId']})
             exc = TaurusConfigError("Need signature for session")
-            self._session.data_signature = self.parameters.get("signature", exc)
+            signature = self.parameters.get("signature", exc)
+            self._session.data_signature = signature
             self._session.kpi_target = self.parameters.get("kpi-target", self._session.kpi_target)
             self.send_data = self.parameters.get("send-data", self.send_data)
         else:
@@ -125,6 +154,26 @@ class BlazeMeterUploader(Reporter, AggregatorListener, MonitoringListener, Singl
                 self._test = finder.resolve_external_test()
             else:
                 self._test = Test(self._user, {'id': None})
+        happysocks_address = self.settings.get("happysocks-address")
+        if happysocks_address and self._sess_id and signature:
+            self.log.info(f"Engine metrics will be sent to '{happysocks_address}'")
+            request_timeout = self.settings.get("happysocks-request-timeout", 10)
+            connect_timeout = self.settings.get("happysocks-connect-timeout", 7)
+            self.happysocks_client = HappysocksClient(happysocks_address, self._sess_id, signature,
+                                                      happysocks_verbose_logging, happysocks_verify_ssl,
+                                                      request_timeout, connect_timeout)
+            if self.engine.config and "settings" in self.engine.config and "proxy" in self.engine.config.get(
+                    "settings"):
+                self.happysocks_client.add_proxy_settings(self.engine.config.get("settings").get("proxy"), force=True)
+        else:
+            reason = ""
+            if not happysocks_address:
+                reason = "modules.blazemeter.happysocks-address setting was not provided"
+            elif not self._sess_id:
+                reason = "Missing session-id parameter in blazemeter reporting module"
+            elif not signature:
+                reason = "Missing signature parameter in blazemeter reporting module"
+            self.log.info(f"Sending of engine metrics to happysocks is disabled. {reason}")
 
         self.report_name = self.parameters.get("report-name", self.settings.get("report-name", self.report_name))
         if self.report_name == 'ask' and sys.stdin.isatty():
@@ -153,6 +202,13 @@ class BlazeMeterUploader(Reporter, AggregatorListener, MonitoringListener, Singl
             if self._user.token and self.public_report:
                 report_link = self._master.make_report_public()
                 self.log.info("Public report link: %s", report_link)
+        if self._metric_reporting_enabled():
+            try:
+                self.happysocks_client.connect()
+                # socket.io client can auto reconnect but only after initial successful connect
+                self._happysocks_auto_reconnect = True
+            except BaseException:
+                self.log.error("Failed to connect to happysocks", exc_info=True)
 
     def _start_online(self):
         """
@@ -162,9 +218,9 @@ class BlazeMeterUploader(Reporter, AggregatorListener, MonitoringListener, Singl
         self.log.info("Initiating data feeding...")
 
         if self._test['id']:
-            self._session, self._master = self._test.start_external()
+            self._session, self._master = self._test.start_external(self.engine.config)
         else:
-            self._session, self._master, self.results_url = self._test.start_anonymous_external_test()
+            self._session, self._master, self.results_url = self._test.start_anonymous_external_test(self.engine.config)
             self._test['id'] = self._session['testId']
 
         if self._test.token:
@@ -257,6 +313,14 @@ class BlazeMeterUploader(Reporter, AggregatorListener, MonitoringListener, Singl
 
             if self.send_monitoring:
                 self.__send_monitoring()
+            if self._metric_reporting_enabled():
+                try:
+                    self._send_engine_metrics_and_concurrency()
+                except BaseException:
+                    self.log.error("Failed to send engine metrics", exc_info=True)
+                finally:
+                    self._happysocks_auto_reconnect = False
+                    self.happysocks_client.disconnect()
         finally:
             self._postproc_phase2()
 
@@ -343,6 +407,13 @@ class BlazeMeterUploader(Reporter, AggregatorListener, MonitoringListener, Singl
 
             if self.send_monitoring:
                 self.__send_monitoring()
+        if self._metric_reporting_enabled() and self._last_metrics_dispatch < (
+                time.time() - self._engine_metrics_send_interval):
+            self._last_metrics_dispatch = time.time()
+            try:
+                self._send_engine_metrics_and_concurrency()
+            except BaseException:
+                self.log.error("Failed to send engine metrics", exc_info=True)
         return super(BlazeMeterUploader, self).check()
 
     def __send_data(self, data, do_check=True, is_final=False):
@@ -359,15 +430,21 @@ class BlazeMeterUploader(Reporter, AggregatorListener, MonitoringListener, Singl
 
     def aggregated_second(self, data):
         """
-        Send online data
+        Send online data and store concurrency data
         :param data: DataPoint
         """
         if self.send_data:
             self.kpi_buffer.append(data)
+        if self._concurrency_reporting_enabled():
+            concurrency_data = HappySocksConcurrencyConverter.extract_concurrency_data(data)
+            if concurrency_data is not None:
+                self._concurrency_buffer.record_data(concurrency_data)
 
-    def monitoring_data(self, data):
+    def monitoring_data(self, data: List[dict]):
         if self.send_monitoring:
             self.monitoring_buffer.record_data(data)
+        if self._metric_reporting_enabled():
+            self._engine_metrics_buffer.record_data(data)
 
     def __send_monitoring(self):
         engine_id = self.engine.config.get('modules').get('shellexec').get('env').get('TAURUS_INDEX_ALL', '')
@@ -385,6 +462,52 @@ class BlazeMeterUploader(Reporter, AggregatorListener, MonitoringListener, Singl
             lines.append(bytestr + " " + fname)
         return "\n".join(lines)
 
+    def _metric_reporting_enabled(self) -> bool:
+        """
+        Returns True if sending reports to happysocks API is enabled.
+        """
+        return self.happysocks_client is not None
+
+    def _concurrency_reporting_enabled(self) -> bool:
+        """
+        Returns True if concurrency reporting is enabled.
+        """
+        return self._metric_reporting_enabled() and self.send_concurrency_metric
+
+    def _send_engine_metrics_and_concurrency(self):
+        """
+        Sends engine metrics and concurrency to happysocks API via WebSockets.
+        """
+        if not self.happysocks_client.connected():
+            if not self._happysocks_auto_reconnect:
+                try:
+                    self.happysocks_client.connect()
+                    # socket.io client can auto reconnect but only after initial successful connect
+                    self._happysocks_auto_reconnect = True
+                except BaseException as ex:
+                    self.log.warning(f"Unable to send engine health metrics to happysocks. {str(ex)}")
+                    return
+            else:
+                # This may happen when the client managed to connect but then connection was dropped. It
+                # may reconnect later. Engine metrics are buffered for a short period.
+                self.log.warning("Unable to send engine health metrics to happysocks. No connection to server.")
+                return
+        raw_data = self._engine_metrics_buffer.get_data()
+        metrics_batch = HappysocksMetricsConverter.to_metrics_batch(raw_data, self._sess_id,
+                                                                    self._master_id,
+                                                                    self._calibration_id, self._calibration_step_id)
+        if metrics_batch:
+            self.happysocks_client.send_engine_metrics(metrics_batch, HappysocksEngineNamespace.METRICS_EVENT)
+        self._engine_metrics_buffer.clear()
+
+        # concurrency
+        raw_concurrency = self._concurrency_buffer.get_data()
+        concurrency_batch = HappySocksConcurrencyConverter.to_concurrency_batch(raw_concurrency,
+                                                                                self._sess_id, self._master_id)
+        if concurrency_batch:
+            self.happysocks_client.send_engine_metrics(concurrency_batch, HappysocksEngineNamespace.CONCURRENCY_EVENT)
+        self._concurrency_buffer.clear()
+
 
 class MonitoringBuffer(object):
     def __init__(self, size_limit, parent_log):
