@@ -30,6 +30,7 @@ from collections import Counter, namedtuple
 from distutils.version import LooseVersion
 from itertools import dropwhile
 from io import StringIO
+from typing import Optional
 
 from cssselect import GenericTranslator
 from lxml import etree
@@ -38,7 +39,7 @@ from bzt import TaurusConfigError, ToolError, TaurusInternalException, TaurusNet
 from bzt.engine import Scenario, ScenarioExecutor
 from bzt.engine import SETTINGS
 from bzt.jmx import JMX, JMeterScenarioBuilder, LoadSettingsProcessor, try_convert
-from bzt.modules.aggregator import ResultsReader, DataPoint, KPISet
+from bzt.modules.aggregator import ResultsReader, DataPoint, KPISet, ErrorResponseData, ERROR_RESPONSE_BODIES_SIZE_LIMIT, ERROR_RESPONSE_BODIES_LIMIT
 from bzt.modules.console import ExecutorWidget
 from bzt.modules.functional import FunctionalResultsReader, FunctionalSample
 from bzt.requests_model import ResourceFilesCollector, has_variable_pattern, HierarchicRequestParser
@@ -189,7 +190,7 @@ class JMeterExecutor(ScenarioExecutor):
                              iterations=iterations, duration=duration, steps=steps)
 
     @staticmethod
-    def _get_tool_version(jmx_file):
+    def _get_tool_version(jmx_file, default_version):
         jmx = JMX(jmx_file)
         selector = 'jmeterTestPlan'
         test_plan = jmx.get(selector)[0]
@@ -204,9 +205,9 @@ class JMeterExecutor(ScenarioExecutor):
             dash_index = ver.find("-")
             if dash_index != -1:
                 # not supported version like: 2.12-SNAPSHOT.20150128
-                return JMeter.VERSION
+                return default_version
             return ver
-        return JMeter.VERSION
+        return default_version
 
     def prepare(self):
         """
@@ -226,22 +227,33 @@ class JMeterExecutor(ScenarioExecutor):
 
         self.original_jmx = self.get_script_path()
         self.log.debug("Getting Jmeter version.")
+        # STABLE is default version for unknown/not present version
+        # if LATEST is not selected as default by version-type
+        default_version =  JMeter.VERSION_LATEST \
+            if self.settings.get("version-type","stable") == 'latest' \
+            else JMeter.VERSION
+        config_version = self.settings.get("version", default_version, force_set=True)
         if os.getenv('TAURUS_JMETER_DISABLE_AUTO_DETECT', 'False') == 'True':
             self.log.debug("Autodetect version is disabled.")
-            # autodetection NOT active -> use STABLE if auto,
+            # autodetection NOT active -> use default_version if auto,
             # xyz if STABLE|LATEST
-            # STABLE otherwise
-            config_version = self.settings.get("version", JMeter.VERSION, force_set=True)
+            # default_version otherwise
             if config_version in [JMeter.VERSION, JMeter.VERSION_LATEST]:
                 self.settings["version"] = config_version
             else:
-                self.settings["version"] = JMeter.VERSION
+                self.settings["version"] = default_version
         else:
             # autodetection IS active (original default behavior)
             self.log.debug("Autodetect version is enabled")
-            if self.settings.get("version", JMeter.VERSION, force_set=True) == "auto":
-                self.settings["version"] = self._get_tool_version(self.original_jmx)
-        self.log.debug("JMeter version: %s", self.settings["version"])
+            if config_version == "auto":
+                self.settings["version"] = self._get_tool_version(self.original_jmx, default_version)
+
+        if self.settings["version"] != config_version:
+            self.log.info("Using JMeter version: %s (configured version: %s, version-type: %s)",
+                          self.settings["version"], config_version,
+                          self.settings.get("version-type", "stable"))
+        else:
+            self.log.debug("JMeter version: %s", self.settings["version"])
 
         if not self.original_jmx:
             if self.get_scenario().get("requests"):
@@ -857,12 +869,45 @@ class JTLReader(ResultsReader):
         self.log = parent_logger.getChild(self.__class__.__name__)
         self.csvreader = IncrementalCSVReader(self.log, filename)
         self.read_records = 0
+        self._collect_error_response_bodies = False
+        self._error_response_bodies_limit = ERROR_RESPONSE_BODIES_LIMIT
+        self._error_response_bodies_size_limit = ERROR_RESPONSE_BODIES_SIZE_LIMIT
         if errors_filename:
             self.errors_reader = JTLErrorsReader(
                 errors_filename, parent_logger, err_msg_separator, label_converter=self.get_mixed_label)
         else:
             self.errors_reader = None
 
+    @property
+    def collect_error_response_bodies(self):
+        return self._collect_error_response_bodies
+
+    @collect_error_response_bodies.setter
+    def collect_error_response_bodies(self, value):
+        self._collect_error_response_bodies = value
+        if self.errors_reader:
+            self.errors_reader.collect_error_response_bodies = value
+
+    @property
+    def error_response_bodies_limit(self):
+        return self._error_response_bodies_limit
+
+    @error_response_bodies_limit.setter
+    def error_response_bodies_limit(self, value):
+        self._error_response_bodies_limit = value
+        if self.errors_reader:
+            self.errors_reader.error_response_bodies_limit = value
+
+    @property
+    def error_response_bodies_size_limit(self):
+        return self._error_response_bodies_size_limit
+
+    @error_response_bodies_size_limit.setter
+    def error_response_bodies_size_limit(self, value):
+        self._error_response_bodies_size_limit = value
+        if self.errors_reader:
+            self.errors_reader.error_response_bodies_size_limit = value
+
     def set_aggregation(self, aggregation):
         super().set_aggregation(aggregation)
         if self.errors_reader:
@@ -1005,15 +1050,6 @@ class FuncJTLReader(FunctionalResultsReader):
             assertions.append({"name": name, "isFailed": failed, "errorMessage": error_message})
         return assertions
 
-    def _parse_http_headers(self, header_str):
-        headers = {}
-        for line in header_str.split("\n"):
-            clean_line = line.strip()
-            if ":" in clean_line:
-                key, value = clean_line.split(":", 1)
-                headers[key] = value
-        return headers
-
     def _parse_http_cookies(self, cookie_str):
         cookies = {}
         clean_line = cookie_str.strip()
@@ -1049,8 +1085,8 @@ class FuncJTLReader(FunctionalResultsReader):
             "threadGroup": thread_group,
 
             "assertions": self._extract_sample_assertions(sample_elem),
-            "requestHeaders": self._parse_http_headers(req_headers),
-            "responseHeaders": self._parse_http_headers(resp_headers),
+            "requestHeaders": JTLReaderUtils.parse_http_headers(req_headers),
+            "responseHeaders": JTLReaderUtils.parse_http_headers(resp_headers),
             "requestCookies": self._parse_http_cookies(req_cookies),
 
             "requestBody": sample_elem.findtext("queryString") or "",
@@ -1138,6 +1174,18 @@ class FuncJTLReader(FunctionalResultsReader):
         return None
 
 
+class JTLReaderUtils:
+    @staticmethod
+    def parse_http_headers(header_str):
+        headers = {}
+        for line in header_str.split("\n"):
+            clean_line = line.strip()
+            if ":" in clean_line:
+                key, value = clean_line.split(":", 1)
+                headers[key] = value
+        return headers
+
+
 class IncrementalCSVReader(object):
     """
     JTL csv reader
@@ -1211,7 +1259,7 @@ class JTLErrorsReader(object):
     url_xpath = GenericTranslator().css_to_xpath("java\\.net\\.URL")
 
     def __init__(self, filename, parent_logger, err_msg_separator=None, label_converter=None):
-        # http://stackoverflow.com/questions/9809469/python-sax-to-lxml-for-80gb-xml/9814580#9814580
+        # https://stackoverflow.com/questions/9809469/python-sax-to-lxml-for-80gb-xml/9814580#9814580
         super(JTLErrorsReader, self).__init__()
         self.log = parent_logger.getChild(self.__class__.__name__)
         self.parser = etree.XMLPullParser(events=('end',))
@@ -1221,6 +1269,10 @@ class JTLErrorsReader(object):
         self.err_msg_separator = err_msg_separator
         self.label_converter = label_converter
         self._redundant_aggregation = False
+        self.collect_error_response_bodies = False
+        self.error_response_bodies_limit = ERROR_RESPONSE_BODIES_LIMIT
+        self.error_response_bodies_size_limit = ERROR_RESPONSE_BODIES_SIZE_LIMIT
+        self._collected_error_responses: set = set()
 
     def set_aggregation(self, aggregation):
         self._redundant_aggregation = aggregation
@@ -1297,7 +1349,7 @@ class JTLErrorsReader(object):
         self._extract_common(elem, label, r_code, t_stamp, message)
 
     def _extract_common(self, elem, label, r_code, t_stamp, r_msg):
-        f_msg, f_url, f_rc, f_tag, f_type = self.find_failure(elem, r_msg, r_code)
+        f_msg, f_url, f_rc, f_tag, f_type, f_response_data = self.find_failure(elem, r_msg, r_code)
 
         if f_type == KPISet.ERRTYPE_SUBSAMPLE:
             url_counts = Counter({f_url: 1})
@@ -1308,13 +1360,24 @@ class JTLErrorsReader(object):
             else:
                 url_counts = Counter()
 
-        err_item = KPISet.error_item_skel(f_msg, f_rc, 1, f_type, url_counts, f_tag)
+        err_item = KPISet.error_item_skel(f_msg, f_rc, 1, f_type, url_counts, f_tag, f_response_data)
         buf = self.buffer.get(t_stamp, force_set=True)
         if self._redundant_aggregation:
             label = self.label_converter(label=label, rc=r_code, msg=r_msg)
+
+        for resp_body in err_item['responseBodies']:
+            if self._should_not_collect_error_response(resp_body['hash']):
+                err_item['responseBodies'].remove(resp_body)
+
         KPISet.inc_list(buf.get(label, [], force_set=True), ("msg", f_msg), err_item)
         KPISet.inc_list(buf.get('', [], force_set=True), ("msg", f_msg), err_item)
 
+    def _should_not_collect_error_response(self, response_hash: int) -> bool:
+        if len(self._collected_error_responses) == self.error_response_bodies_limit:
+            return True
+        self._collected_error_responses.add(response_hash)
+        return False
+
     def _extract_nonstandard(self, elem):
         t_stamp = int(elem.findtext("timeStamp")) / 1000.0
         label = elem.findtext("label")
@@ -1324,16 +1387,20 @@ class JTLErrorsReader(object):
         self._extract_common(elem, label, r_code, t_stamp, message)
 
     def find_failure(self, element, def_msg="", def_rc=None):
-        """ returns (message, url, rc, tag, err_type) """
+        """ returns (message, url, rc, tag, err_type, err_response_data) """
         rc = element.get("rc", default="")
 
         e_msg = ""
         url = None
         err_type = KPISet.ERRTYPE_ERROR
 
+        err_response_data: Optional[ErrorResponseData] = None
+        if self.collect_error_response_bodies:
+            err_response_data = self._get_response_data(element)
+
         a_msg, name = get_child_assertion(element)
 
-        if not rc.startswith("2"):  # this sample is failed
+        if not rc.startswith("2") and not rc.startswith("3"):  # this sample is failed
             e_msg = element.get("rm", default="")
             url = element.xpath(self.url_xpath)
             url = url[0].text if url else element.get("lb")
@@ -1342,7 +1409,7 @@ class JTLErrorsReader(object):
         elif element.get("s") == "false":  # has failed sub element, we should look deeper...
             for child in element.iterchildren():
                 if child.tag in ("httpSample", "sample"):  # let's check sub samples..
-                    e_msg, url, rc, name, err_type = self.find_failure(child)
+                    e_msg, url, rc, name, err_type, err_response_data = self.find_failure(child)
                     if e_msg:
                         if err_type == KPISet.ERRTYPE_ERROR:  # replace subsample error
                             err_type = KPISet.ERRTYPE_SUBSAMPLE
@@ -1360,7 +1427,33 @@ class JTLErrorsReader(object):
             if self.err_msg_separator:  # add appropriate separator to default msg
                 msg = self.err_msg_separator + msg
 
-        return msg, url, rc or def_rc, name, err_type
+        return msg, url, rc or def_rc, name, err_type, err_response_data
+
+    def _get_response_data(self, element) -> Optional[ErrorResponseData]:
+        content_type = self._get_response_content_type(element)
+        content, content_size = self._get_response_content(element)
+        if content is not None:
+            return ErrorResponseData(content, content_type, content_size)
+
+    def _get_response_content(self, element) -> (str, int):
+        for child in element.iterchildren():
+            if child.tag == 'responseData' and child.text is not None:
+                body_size = len(child.text)
+                body_content = child.text[:self.error_response_bodies_size_limit]
+                return body_content, body_size
+        return '', 0
+
+    @staticmethod
+    def _get_response_content_type(element) -> Optional[str]:
+        headers_text: Optional[str] = None
+        for child in element.iterchildren():
+            if child.tag == 'responseHeader':
+                headers_text = child.text
+                break
+
+        if headers_text is not None:
+            headers = JTLReaderUtils.parse_http_headers(headers_text)
+            return headers['Content-Type'] if 'Content-Type' in headers else None
 
 
 class XMLJTLReader(JTLErrorsReader, ResultsReader):
@@ -1394,18 +1487,23 @@ class JMeter(RequiredTool):
     """
     JMeter tool
     """
-    PLUGINS_MANAGER_VERSION = "1.7"
+    PLUGINS_MANAGER_VERSION = "1.10"
     PLUGINS_MANAGER_LINK = 'https://search.maven.org/remotecontent?filepath=kg/apc/jmeter-plugins-manager/{version}/jmeter-plugins-manager-{version}.jar'
-    COMMAND_RUNNER_VERSION = "2.2"
+    COMMAND_RUNNER_VERSION = "2.3"
     COMMAND_RUNNER_LINK = 'https://search.maven.org/remotecontent?filepath=kg/apc/cmdrunner/{version}/cmdrunner-{version}.jar'
-    VERSION = "5.4.3"
-    VERSION_LATEST = "5.5"
+    VERSION = "5.5"
+    VERSION_LATEST = "5.6.3"
 
     def __init__(self, config=None, props=None, **kwargs):
         settings = config or BetterDict()
         props = props or BetterDict()
 
-        version = settings.get("version", JMeter.VERSION)
+        # STABLE is default version for unknown/not present version
+        # if LATEST is not selected as default by version-type
+        default_version =  JMeter.VERSION_LATEST \
+            if settings.get("version-type","stable") == 'latest' \
+            else JMeter.VERSION
+        version = settings.get("version", default_version)
         jmeter_path = settings.get("path", "~/.bzt/jmeter-taurus/{version}/")
         jmeter_path = get_full_path(jmeter_path).format(version=version)
 
@@ -1582,17 +1680,19 @@ class JMeter(RequiredTool):
         affected_components = {
             # Needs to be <1.4.18 for old Jmeters https://stackoverflow.com/questions/30812293/com-thoughtworks-xstream-security-forbiddenclassexception
             "xstream": "com/thoughtworks/xstream/xstream/1.4.20/xstream-1.4.20.jar",
-            "jackson-annotations": "com/fasterxml/jackson/core/jackson-annotations/2.14.1/jackson-annotations-2.14.1.jar",
-            "jackson-core": "com/fasterxml/jackson/core/jackson-core/2.14.1/jackson-core-2.14.1.jar",
-            "jackson-databind": "com/fasterxml/jackson/core/jackson-databind/2.14.1/jackson-databind-2.14.1.jar",
+            "jackson-annotations": "com/fasterxml/jackson/core/jackson-annotations/2.15.0/jackson-annotations-2.15.0.jar",
+            "jackson-core": "com/fasterxml/jackson/core/jackson-core/2.15.0/jackson-core-2.15.0.jar",
+            "jackson-databind": "com/fasterxml/jackson/core/jackson-databind/2.15.0/jackson-databind-2.15.0.jar",
             "json-smart": "net/minidev/json-smart/2.4.8/json-smart-2.4.8.jar",
             "jsoup": "org/jsoup/jsoup/1.15.3/jsoup-1.15.3.jar",
-            "snakeyaml": "org/yaml/snakeyaml/1.33/snakeyaml-1.33.jar",
+            "snakeyaml": "org/yaml/snakeyaml/2.0/snakeyaml-2.0.jar",
             "okhttp": "com/squareup/okhttp3/okhttp/4.10.0/okhttp-4.10.0.jar",
             "commons-text": "org/apache/commons/commons-text/1.10.0/commons-text-1.10.0.jar",
-            "xmlgraphics-commons": "org/apache/xmlgraphics/xmlgraphics-commons/2.8/xmlgraphics-commons-2.8.jar"}
+            "xmlgraphics-commons": "org/apache/xmlgraphics/xmlgraphics-commons/2.8/xmlgraphics-commons-2.8.jar",
+            "xalan-2.7.2": "xalan/xalan/2.7.3/xalan-2.7.3.jar"
+        }
 
-        if LooseVersion(self.version) <= LooseVersion('5.4.2'):  # log4j must be fixed till jmeter 5.4.2
+        if LooseVersion(self.version) <= LooseVersion('5.4.3'):  # log4j must be fixed till jmeter 5.4.3
             affected_names = ["log4j-core", "log4j-api", "log4j-slf4j-impl", "log4j-1.2-api"]
             fixed_version = '2.19.0'
             maven_link = "org/apache/logging/log4j/{name}/{ver}/{name}-{ver}.jar"
@@ -1702,6 +1802,7 @@ class JarCleaner(object):
 class JMeterMirrorsManager(MirrorsManager):
     MIRRORS_SOURCE = "https://jmeter.apache.org/download_jmeter.cgi"
     DOWNLOAD_LINK = "https://archive.apache.org/dist/jmeter/binaries/apache-jmeter-{version}.zip"
+    BZM_REGISTRY_LINK = "https://packages.blazemeter.com/jmeter/apache-jmeter-{version}.zip"
 
     def __init__(self, http_client, parent_logger, jmeter_version):
         self.jmeter_version = jmeter_version
@@ -1719,6 +1820,7 @@ class JMeterMirrorsManager(MirrorsManager):
                 link_tail = "/jmeter/binaries/apache-jmeter-{version}.zip".format(version=self.jmeter_version)
                 links = [link.strip('<option value="').strip('">') + link_tail for link in option_elements]
         links.append(self.DOWNLOAD_LINK.format(version=self.jmeter_version))
+        links.append(self.BZM_REGISTRY_LINK.format(version=self.jmeter_version))
         self.log.debug('Total mirrors: %d', len(links))
         # place HTTPS links first, preserving the order of HTTP links
         sorted_links = sorted(links, key=lambda l: l.startswith("https"), reverse=True)
