@@ -104,14 +104,22 @@ class K6LogReader(ResultsReader):
         self.log = parent_logger.getChild(self.__class__.__name__)
         self.file = FileReader(filename=filename, parent_logger=self.log)
         self.data = {'timestamp': [], 'label': [], 'r_code': [], 'error_msg': [], 'http_req_duration': [],
-                     'http_req_connecting': [], 'http_req_tls_handshaking': [], 'http_req_waiting': [], 'vus': [],
+                     'http_req_connecting': [], 'http_req_tls_handshaking': [], 'http_req_waiting': [],
                      'data_received': []}
         self.position = {'timestamp': None, 'metric_value': None, 'error': None,
                          'expected_response': None, 'name': None, 'status': None}
 
     def _read(self, last_pass=False):
         self.lines = list(self.file.get_lines(size=1024 * 1024, last_pass=last_pass))
-
+        self.calculate = False
+        self.vus = -1
+        for line in self.lines:
+            if line.startswith("metric_name"):
+                parts = line[:-1].split(",")
+                self.position['timestamp'] = parts.index('timestamp')
+            elif line.startswith("http_reqs"):
+                self.previous_timestamp = int(line.split(',')[self.position['timestamp']])
+                break
         for line in self.lines:
             if line.startswith("metric_name"):
                 parts = line[:-1].split(",")
@@ -122,48 +130,89 @@ class K6LogReader(ResultsReader):
                 self.position['name'] = parts.index('name')
                 self.position['status'] = parts.index('status')
             elif line.startswith("http_reqs"):
-                self.data['timestamp'].append(int(line.split(',')[self.position['timestamp']]))
-                self.data['label'].append(line.split(',')[self.position['name']])
-                self.data['r_code'].append(line.split(',')[self.position['status']])
-                error = line.split(',')[self.position['error']]
-                if not error and line.split(',')[self.position['expected_response']] == 'false':
-                    error = f"Response code: {line.split(',')[self.position['status']]}"
-                self.data['error_msg'].append(error)
+                parts = line.split(",")
+                if len(parts) > self.position['status']:
+                    current_timestamp = int(parts[self.position['timestamp']])
+                    if current_timestamp > self.previous_timestamp:
+                        yield from self.calculate_timestamp_data()
+                        self.previous_timestamp = current_timestamp
+                    self.data['timestamp'].append(current_timestamp)
+                    self.data['label'].append(parts[self.position['name']])
+                    self.data['r_code'].append(parts[self.position['status']])
+                    error = parts[self.position['error']]
+                    if not error and parts[self.position['expected_response']] == 'false':
+                        error = f"Response code: {parts[self.position['status']]}"
+                    self.data['error_msg'].append(error)
             elif line.startswith("http_req_duration"):
-                self.data['http_req_duration'].append(float(line.split(',')[self.position['metric_value']]))
+                parts = line.split(",")
+                if len(parts) > self.position['status']:
+                    self.data['http_req_duration'].append(float(parts[self.position['metric_value']]))
             elif line.startswith("http_req_connecting"):
-                self.data['http_req_connecting'].append(float(line.split(',')[self.position['metric_value']]))
+                parts = line.split(",")
+                if len(parts) > self.position['status']:
+                    self.data['http_req_connecting'].append(float(parts[self.position['metric_value']]))
             elif line.startswith("http_req_tls_handshaking"):
-                self.data['http_req_tls_handshaking'].append(float(line.split(',')[self.position['metric_value']]))
+                parts = line.split(",")
+                if len(parts) > self.position['status']:
+                    self.data['http_req_tls_handshaking'].append(float(parts[self.position['metric_value']]))
             elif line.startswith("http_req_waiting"):
-                self.data['http_req_waiting'].append(float(line.split(',')[self.position['metric_value']]))
-            elif line.startswith("vus") and not line.startswith("vus_max"):
-                self.data['vus'].append(int(float(line.split(',')[self.position['metric_value']])))
+                parts = line.split(",")
+                if len(parts) > self.position['status']:
+                    self.data['http_req_waiting'].append(float(parts[self.position['metric_value']]))
+            elif line.startswith("vus") and not line.startswith("vus_max") and self.vus == -1:
+                parts = line.split(",")
+                if len(parts) > self.position['status']:
+                    self.vus = (int(float(parts[self.position['metric_value']])))
             elif line.startswith("data_received"):
-                self.data['data_received'].append(float(line.split(',')[self.position['metric_value']]))
+                parts = line.split(",")
+                if len(parts) > self.position['status']:
+                    self.data['data_received'].append(float(parts[self.position['metric_value']]))
 
-            if self.data['vus'] and len(self.data['data_received']) >= self.data['vus'][0] and \
-                    len(self.data['http_req_waiting']) >= self.data['vus'][0]:
-                for i in range(self.data['vus'][0]):
-                    kpi_set = (
-                        self.data['timestamp'][0],
-                        self.data['label'][0],
-                        self.data['vus'][0],
-                        self.data['http_req_duration'][0] / 1000,
-                        (self.data['http_req_connecting'][0] + self.data['http_req_tls_handshaking'][0]) / 1000,
-                        self.data['http_req_waiting'][0] / 1000,
-                        self.data['r_code'][0],
-                        None if not self.data['error_msg'][0] else self.data['error_msg'][0],
-                        '',
-                        self.data['data_received'][0])
+        yield from self.calculate_timestamp_data()
 
-                    for key in self.data.keys():
-                        if key != 'vus':
-                            self.data[key].pop(0)
+    def calculate_timestamp_data(self):
+        # calculate bandwidth
+        bandwidth = 0
+        if self.data['data_received']:
+            for i in range(len(self.data['data_received'])):
+                bandwidth = bandwidth + self.data['data_received'][i]
+        if self.data['timestamp']:
+            self.data
 
-                    yield kpi_set
+            for i in range(len(self.data['timestamp'])):
+                # fill missing matrix items
+                timestamp_length = len(self.data['timestamp'])
+                if timestamp_length > len(self.data['label']):
+                    self.data['label'] += [''] * (timestamp_length - len(self.data['label']))
+                if timestamp_length > len(self.data['http_req_duration']):
+                    self.data['http_req_duration'] += [0] * (timestamp_length - len(self.data['http_req_duration']))
+                if timestamp_length > len(self.data['http_req_connecting']):
+                    self.data['http_req_connecting'] += [0] * (timestamp_length - len(self.data['http_req_connecting']))
+                if timestamp_length > len(self.data['http_req_tls_handshaking']):
+                    self.data['http_req_tls_handshaking'] += [0] * (
+                                timestamp_length - len(self.data['http_req_tls_handshaking']))
+                if timestamp_length > len(self.data['http_req_waiting']):
+                    self.data['http_req_waiting'] += [0] * (timestamp_length - len(self.data['http_req_waiting']))
+                if timestamp_length > len(self.data['r_code']):
+                    self.data['r_code'] += [0] * (timestamp_length - len(self.data['r_code']))
+                if timestamp_length > len(self.data['error_msg']):
+                    self.data['error_msg'] += [None] * (timestamp_length - len(self.data['error_msg']))
+                kpi_set = (
+                    self.data['timestamp'][i],
+                    self.data['label'][i],
+                    self.vus,
+                    self.data['http_req_duration'][i] / 1000,
+                    (self.data['http_req_connecting'][i] + self.data['http_req_tls_handshaking'][i]) / 1000,
+                    self.data['http_req_waiting'][i] / 1000,
+                    self.data['r_code'][i],
+                    None if not self.data['error_msg'][i] else self.data['error_msg'][i],
+                    '',
+                    bandwidth)
 
-                self.data['vus'].pop(0)
+                yield kpi_set
+        self.data = {'timestamp': [], 'label': [], 'r_code': [], 'error_msg': [], 'http_req_duration': [],
+                     'http_req_connecting': [], 'http_req_tls_handshaking': [], 'http_req_waiting': [],
+                     'data_received': []}
 
 
 class K6(RequiredTool):
