@@ -27,7 +27,7 @@ from urwid import Pile, Text
 from bzt import AutomatedShutdown, TaurusConfigError
 from bzt.engine import Reporter, EngineModule
 from bzt.modules.aggregator import KPISet, DataPoint, AggregatorListener, ResultsProvider
-from bzt.modules.console import WidgetProvider, PrioritizedWidget
+from bzt.modules.console import PrioritizedWidget
 from bzt.utils import viewvalues, iteritems, load_class, dehumanize_time, get_bytes_count, BetterDict
 
 
@@ -101,7 +101,7 @@ class CriteriaProcessor(AggregatorListener):
                         raise AutomatedShutdown("%s" % crit)
 
 
-class PassFailStatus(Reporter, WidgetProvider):
+class PassFailStatus(Reporter):
     """
     :type processors: list[CriteriaProcessor]
     """
@@ -279,7 +279,7 @@ class FailCriterion(object):
         if logic == 'for':
             return lambda tstmp, value: value
         elif logic in ('within', 'over'):
-            return self._within_aggregator_avg  # FIXME: having simple average for percented values is a bit wrong
+            return self._within_aggregator_avg
         else:
             raise TaurusConfigError("Unsupported window logic: %s" % logic)
 
