@@ -98,7 +98,10 @@ class FinalStatus(Reporter, AggregatorListener, FunctionalAggregatorListener):
                     self.__report_percentiles(summary_kpi)
 
             if self.parameters.get("summary-labels", True):
-                self.__report_summary_labels(self.last_sec[DataPoint.CUMULATIVE])
+                if isinstance(self.engine.provisioning, CloudProvisioning):
+                    self.__report_summary_labels(self.last_sec[DataPoint.CUMULATIVE], True)
+                else:
+                    self.__report_summary_labels(self.last_sec[DataPoint.CUMULATIVE], False)
 
             if self.parameters.get("failed-labels"):
                 self.__report_failed_labels(self.last_sec[DataPoint.CUMULATIVE])
@@ -107,7 +110,10 @@ class FinalStatus(Reporter, AggregatorListener, FunctionalAggregatorListener):
                 self.__dump_xml(self.parameters.get("dump-xml"))
 
             if self.parameters.get("dump-csv"):
-                self.__dump_csv(self.parameters.get("dump-csv"))
+                if isinstance(self.engine.provisioning, CloudProvisioning):
+                    self.__dump_cloud_csv(self.engine.provisioning.router.master, self.parameters.get("dump-csv"))
+                else:
+                    self.__dump_csv(self.parameters.get("dump-csv"))
         elif self.cumulative_results:
             self.__report_summary()
 
@@ -203,7 +209,7 @@ class FinalStatus(Reporter, AggregatorListener, FunctionalAggregatorListener):
             "\n".join(errors)
         )
 
-    def __report_summary_labels(self, cumulative):
+    def __report_summary_labels(self, cumulative, include_all_label):
         data = [("label", "status", "succ", "avg_rt", "error")]
         justify = {0: "left", 1: "center", 2: "right", 3: "right", 4: "left"}
 
@@ -211,6 +217,8 @@ class FinalStatus(Reporter, AggregatorListener, FunctionalAggregatorListener):
         for sample_label in sorted_labels:
             if sample_label != "":
                 data.append(self.__get_sample_element(cumulative[sample_label], sample_label))
+            elif include_all_label:
+                data.append(self.__get_sample_element(cumulative[''], 'ALL'))
         table = AsciiTable(data)
         table.justify_columns = justify
         self.log.info("Request label stats:\n%s", table.table)
@@ -297,6 +305,51 @@ class FinalStatus(Reporter, AggregatorListener, FunctionalAggregatorListener):
                 for label, kpiset in iteritems(self.last_sec[DataPoint.CUMULATIVE]):
                     writer.writerow(self.__get_csv_dict(label, kpiset))
 
+    def __dump_cloud_csv(self, master, filename):
+        report = master.get_csv_report(filename)
+        self.log.info("Dumping final cloud status as CSV: %s", filename)
+        with open(get_full_path(filename), 'wt') as fhd:
+            writer = csv.writer(fhd)
+            for table in report:
+                csv_lines = str(report[table]).splitlines()
+                csv_lines_cells = []
+                # iterate and set ALL label
+                for line in csv_lines:
+                    cells = line.split(',')
+                    if cells and not cells[0]:
+                        cells[0] = 'ALL'
+                    csv_lines_cells.append(cells)
+                # sort ALL to the top
+                self.__move_string_starting_with_all_to_front(csv_lines_cells)
+                # write csv
+                has_title = False
+                for cells in csv_lines_cells:
+                    if not has_title:
+                        header = [table]
+                        for _ in cells:
+                            header.append(None)
+                        writer.writerow(header)
+                        has_title = True
+                    writer.writerow(cells)
+
+    def __move_string_starting_with_all_to_front(self, cells_array):
+        if cells_array is None:
+            return None
+        if not cells_array:
+            return []
+        if len(cells_array) == 1:
+            return cells_array;
+        all_labeled = None
+        for idx, row in enumerate(cells_array):
+            if row and row[0] and row[0].startswith("ALL"):
+                all_labeled = cells_array.pop(idx)
+                break
+
+        if all_labeled is not None:
+            cells_array.insert(1, all_labeled)
+
+        return cells_array
+
     def __get_csv_dict(self, label, kpiset):
         kpi_copy = copy.deepcopy(kpiset)
         res = OrderedDict()
