@@ -21,14 +21,14 @@ import time
 from math import ceil
 
 from bzt import TaurusConfigError, ToolError
-from bzt.engine import ScenarioExecutor, FileLister, HavingInstallableTools, SelfDiagnosable
+from bzt.engine import ScenarioExecutor
 from bzt.modules.aggregator import ConsolidatingAggregator, ResultsReader
-from bzt.modules.console import WidgetProvider, ExecutorWidget
+from bzt.modules.console import ExecutorWidget
 from bzt.requests_model import HTTPRequest
 from bzt.utils import iteritems, CALL_PROBLEMS, shutdown_process, RequiredTool, dehumanize_time, FileReader
 
 
-class SiegeExecutor(ScenarioExecutor, WidgetProvider, HavingInstallableTools, FileLister, SelfDiagnosable):
+class SiegeExecutor(ScenarioExecutor):
     def __init__(self):
         super(SiegeExecutor, self).__init__()
         self.log = logging.getLogger('')
@@ -124,6 +124,10 @@ class SiegeExecutor(ScenarioExecutor, WidgetProvider, HavingInstallableTools, Fi
         for key, val in iteritems(self.scenario.get_headers()):
             args += ['--header', "%s: %s" % (key, val)]
 
+        user_cmd = self.settings.get("cmdline")
+        if user_cmd:
+            args += user_cmd.split(" ")
+
         self.env.set({"SIEGERC": self.__rc_name})
         self.process = self._execute(args)
 
@@ -137,11 +141,8 @@ class SiegeExecutor(ScenarioExecutor, WidgetProvider, HavingInstallableTools, Fi
 
     def get_widget(self):
         if not self.widget:
-            if self.get_load().hold:
-                label = "Siege Benchmark"
-            else:
-                label = None
-            self.widget = ExecutorWidget(self, label)
+            label = "%s" % self
+            self.widget = ExecutorWidget(self, "Siege: " + label.split('/')[1])
         return self.widget
 
     def shutdown(self):
