@@ -1,7 +1,9 @@
 # Utility functions and classes for Taurus Selenium tests
+import numbers
 import time
-
-from apiritif import get_transaction_handlers, set_transaction_handlers, get_from_thread_store, get_iteration, external_handler
+import logging
+from apiritif import get_transaction_handlers, set_transaction_handlers, get_from_thread_store, get_iteration, \
+    external_handler, CSVReaderPerThread
 
 from selenium.common.exceptions import NoSuchWindowException, NoSuchFrameException, NoSuchElementException, \
     TimeoutException, UnexpectedAlertPresentException
@@ -22,11 +24,13 @@ BYS = {
     'linktext': By.LINK_TEXT
 }
 
+log = logging.getLogger(__name__)
 
 def find_element_by_shadow(shadow_loc):
     """
     Enables finding element using the Shadow Locator
-    :param shadow_loc: shadow locator in the string form - css locators divided by commas, e.g. 'c-comp1, c-basic, .std_btn'
+    :param shadow_loc: shadow locator in the string form - css locators divided by commas, e.g. 'c-comp1, c-basic,
+    .std_btn'
     :return: the found element otherwise NoSuchElementException is raised
     """
     el = None
@@ -169,7 +173,14 @@ def add_flow_markers():
 
 
 def _send_marker(stage, params):
-    _get_driver().execute_script("/* FLOW_MARKER test-case-%s */" % stage, params)
+    for attempt in range(3):
+        try:
+            _get_driver().execute_script("/* FLOW_MARKER test-case-%s */" % stage, params)
+            return
+        except Exception as exc:
+            log.error(f"Failed to send flow marker: {stage} {params} due to {exc}. Retrying... attempt = {attempt + 1}")
+            time.sleep(1)
+    log.error(f"Failed to send flow marker: {stage} {params} due to {exc} after 3 attempts.")
 
 
 def _send_start_flow_marker(*args, **kwargs):  # for apiritif. remove when compatibiltiy code in
@@ -328,7 +339,7 @@ def presence_of_shadow_element_located(locator):
     """
     Extends expected_conditions.py to support shadow locators
     """
-    def _predicate(driver=None):
+    def _predicate():
         return find_element_by_shadow(locator)
 
     return _predicate
@@ -533,3 +544,46 @@ def action_start(action):
 def action_end(action):
     driver = _get_driver()
     external_handler(driver.session_id if driver else None, BaseActionHandler.YAML_ACTION_END, action)
+
+
+def get_data_source(source):
+    filename = source.get("path")
+    fieldnames = source.get("variable-names", None)
+    quoted = source.get("quoted", None)
+    delimiter = source.get("delimiter", None)
+    encoding = source.get("encoding", None)
+
+    return filename, fieldnames, quoted, delimiter, encoding
+
+
+def get_csv_reader_for_entity_loop(data_sources, entity_name):
+    filename = None
+    fieldnames = None
+    quoted = False
+    delimiter = None
+    encoding = None
+    for idx, source in enumerate(data_sources, start=1):
+        if source.get("path", "").lower().endswith(entity_name.lower() + ".csv") \
+                or source.get("path", "").lower().endswith(entity_name.lower()):
+            filename, fieldnames, quoted, delimiter, encoding = get_data_source(source)
+
+    if not filename:
+        # if there is just one CSV try to use it (case for scriptless default entity)
+        if len(data_sources) == 1:
+            filename, fieldnames, quoted, delimiter, encoding = get_data_source(data_sources[0])
+        else:
+            raise TaurusException("Data entity name '{}' for 'loopOverData' not found in 'data-sources'!".format(entity_name))
+
+    # fieldnames can't be empty array otherwise the csv reader will not read the header
+    if isinstance(fieldnames, list) and len(fieldnames) == 0:
+        fieldnames = None
+
+    return CSVReaderPerThread(filename=filename, fieldnames=fieldnames, loop=False, quoted=quoted, delimiter=delimiter,
+                              encoding=encoding)
+
+
+def get_current_iteration():
+    current = get_from_thread_store("current_iteration")
+    if current is not None and isinstance(current, numbers.Integral):
+        return current + 1
+    return 0
